@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: geany.h 4929 2010-05-16 18:22:24Z eht16 $
+ * $Id: geany.h 5023 2010-06-12 13:42:50Z eht16 $
  */
 
 /* Main header - should be included first in all source files.
@@ -47,7 +47,7 @@
  * listed in the documentation should not be changed */
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
 #define GEANY_TEMPLATES_SUBDIR			"templates"
-#define GEANY_CODENAME					"Vellam"
+#define GEANY_CODENAME					"Disra"
 #define GEANY_HOMEPAGE					"http://www.geany.org/"
 #define GEANY_STRING_UNTITLED			_("untitled")
 #define GEANY_DEFAULT_DIALOG_HEIGHT		350
