@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugindata.h 5005 2010-06-06 18:34:26Z eht16 $
+ * $Id: plugindata.h 5497 2010-12-22 16:59:38Z ntrel $
  */
 
 /**
@@ -43,34 +43,41 @@
 #include "editor.h"	/* GeanyIndentType */
 
 
-/* Note: We use enum instead of 'static const gint' to allow its use in global variable
- * initializing, otherwise we get errors like:
- * error: initializer element is not constant */
-/** Versioning data */
-enum {
-	/** The Application Programming Interface (API) version, incremented
-	 * whenever any plugin data types are modified or appended to. */
-	GEANY_API_VERSION = 189,
+/** The Application Programming Interface (API) version, incremented
+ * whenever any plugin data types are modified or appended to.
+ *
+ * You can protect code that needs a higher API than e.g. 200 with:
+ * @code #if GEANY_API_VERSION >= 200
+ * 	some_newer_function();
+ * #endif @endcode
+ *
+ * @warning You should not test for values below 200 as previously
+ * @c GEANY_API_VERSION was defined as an enum value, not a macro.
+ */
+#define GEANY_API_VERSION 201
 
-	/** The Application Binary Interface (ABI) version, incremented whenever
-	 * existing fields in the plugin data types have to be changed or reordered. */
-	/* This should usually stay the same if fields are only appended, assuming only pointers to
-	 * structs and not structs themselves are declared by plugins. */
-	GEANY_ABI_VERSION = 66
-};
+/** The Application Binary Interface (ABI) version, incremented whenever
+ * existing fields in the plugin data types have to be changed or reordered.
+ * Changing this forces all plugins to be recompiled before Geany can load them. */
+/* This should usually stay the same if fields are only appended, assuming only pointers to
+ * structs and not structs themselves are declared by plugins. */
+#define GEANY_ABI_VERSION 68
 
-/** Checks the plugin can be loaded by Geany.
+
+/** Defines a function to check the plugin is safe to load.
  * This performs runtime checks that try to ensure:
  * - Geany ABI data types are compatible with this plugin.
- * - Geany sources provide the required API for this plugin. */
+ * - Geany sources provide the required API for this plugin.
+ * @param api_required The minimum API number your plugin requires.
+ * Look at the source for the value of @c GEANY_API_VERSION to use if you
+ * want your plugin to require the current Geany version on your machine.
+ * You should update this value when using any new API features. */
 #define PLUGIN_VERSION_CHECK(api_required) \
 	gint plugin_version_check(gint abi_ver) \
 	{ \
 		if (abi_ver != GEANY_ABI_VERSION) \
 			return -1; \
-		if (GEANY_API_VERSION < (api_required)) \
-			return (api_required); \
-		else return 0; \
+		return (api_required); \
 	}
 
 
@@ -111,7 +118,7 @@ GeanyPlugin;
  * extra PluginInfo features (such as an icon), so we don't need to break API
  * compatibility. Alternatively just add a new macro, PLUGIN_SET_INFO_FULL(). -ntrel */
 #define PLUGIN_SET_INFO(p_name, p_description, p_version, p_author) \
-	void plugin_set_info(PluginInfo* info) \
+	void plugin_set_info(PluginInfo *info) \
 	{ \
 		info->name = (p_name); \
 		info->description = (p_description); \
@@ -129,15 +136,25 @@ GeanyPlugin;
  *
  * @since 0.19 */
 #define PLUGIN_SET_TRANSLATABLE_INFO(localedir, package, p_name, p_description, p_version, p_author) \
-	void plugin_set_info(PluginInfo* info) \
+	void plugin_set_info(PluginInfo *info) \
 	{ \
-		main_locale_init(localedir, package); \
+		main_locale_init((localedir), (package)); \
 		info->name = (p_name); \
 		info->description = (p_description); \
 		info->version = (p_version); \
 		info->author = (p_author); \
 	}
 
+
+#ifndef GEANY_PRIVATE
+
+/* Prototypes for building plugins with -Wmissing-prototypes */
+gint plugin_version_check(gint abi_ver);
+void plugin_set_info(PluginInfo *info);
+
+#endif
+
+
 /** @deprecated - use plugin_set_key_group() instead.
  * @see PLUGIN_KEY_GROUP() macro. */
 typedef struct GeanyKeyGroupInfo
@@ -171,7 +188,7 @@ GeanyKeyGroupInfo;
 typedef struct PluginCallback
 {
 	/** The name of signal, must be an existing signal. For a list of available signals,
-	 *  please see the @link signals Signal documentation @endlink. */
+	 *  please see the @link pluginsignals.c Signal documentation @endlink. */
 	const gchar	*signal_name;
 	/** A callback function which is called when the signal is emitted. */
 	GCallback	callback;
@@ -303,7 +320,7 @@ struct _ScintillaObject;
 typedef struct ScintillaFuncs
 {
 	/** Send Scintilla a message. */
-	long int	(*scintilla_send_message) (struct _ScintillaObject* sci, unsigned int iMessage,
+	long int	(*scintilla_send_message) (struct _ScintillaObject *sci, unsigned int iMessage,
 			long unsigned int wParam, long int lParam);
 	/** Create a new ScintillaObject widget. */
 	GtkWidget*	(*scintilla_new)(void);
@@ -371,6 +388,7 @@ typedef struct SciFuncs
 	gint	(*sci_find_text) (struct _ScintillaObject *sci, gint flags, struct Sci_TextToFind *ttf);
 	void	(*sci_set_line_indentation) (struct _ScintillaObject *sci, gint line, gint indent);
 	gint	(*sci_get_line_indentation) (struct _ScintillaObject *sci, gint line);
+	gint	(*sci_get_lexer) (struct _ScintillaObject *sci);
 }
 SciFuncs;
 
@@ -417,7 +435,7 @@ typedef struct UtilsFuncs
 	GSList*		(*utils_get_file_list_full)(const gchar *path, gboolean full_path, gboolean sort,
 				GError **error);
 	gchar**		(*utils_copy_environment)(const gchar **exclude_vars, const gchar *first_varname, ...);
-
+	gchar*		(*utils_find_open_xml_tag) (const gchar sel[], gint size);
 }
 UtilsFuncs;
 
@@ -453,6 +471,8 @@ typedef struct UIUtilsFuncs
 	void		(*ui_widget_modify_font_from_string) (GtkWidget *widget, const gchar *str);
 	gboolean	(*ui_is_keyval_enter_or_return) (guint keyval);
 	gint		(*ui_get_gtk_settings_integer) (const gchar *property_name, gint default_value);
+	void		(*ui_combo_box_add_to_history) (GtkComboBoxEntry *combo_entry,
+				const gchar *text, gint history_len);
 }
 UIUtilsFuncs;
 
@@ -465,6 +485,8 @@ typedef struct DialogFuncs
 	gboolean	(*dialogs_show_save_as) (void);
 	gboolean	(*dialogs_show_input_numeric) (const gchar *title, const gchar *label_text,
 				 gdouble *value, gdouble min, gdouble max, gdouble step);
+	gchar*		(*dialogs_show_input)(const gchar *title, GtkWindow *parent, const gchar *label_text,
+				const gchar *default_text);
 }
 DialogFuncs;
 
@@ -487,6 +509,7 @@ typedef struct MsgWinFuncs
 				 const gchar *format, ...) G_GNUC_PRINTF (4, 5);
 	void		(*msgwin_clear_tab) (gint tabnum);
 	void		(*msgwin_switch_tab) (gint tabnum, gboolean show);
+	void		(*msgwin_set_messages_dir) (const gchar *messages_dir);
 }
 MsgWinFuncs;
 
@@ -524,6 +547,9 @@ typedef struct HighlightingFuncs
 {
 	const struct GeanyLexerStyle* (*highlighting_get_style) (gint ft_id, gint style_id);
 	void		(*highlighting_set_styles) (struct _ScintillaObject *sci, struct GeanyFiletype *ft);
+	gboolean	(*highlighting_is_string_style) (gint lexer, gint style);
+	gboolean	(*highlighting_is_comment_style) (gint lexer, gint style);
+	gboolean	(*highlighting_is_code_style) (gint lexer, gint style);
 }
 HighlightingFuncs;
 
@@ -534,6 +560,7 @@ typedef struct FiletypeFuncs
 	GeanyFiletype*	(*filetypes_detect_from_file) (const gchar *utf8_filename);
 	GeanyFiletype*	(*filetypes_lookup_by_name) (const gchar *name);
 	GeanyFiletype*	(*filetypes_index)(gint idx);
+	const gchar*	(*filetypes_get_display_name)(GeanyFiletype *ft);
 	/* Remember to convert any filetype_id arguments to GeanyFiletype pointers in any
 	 * appended functions */
 }
@@ -593,6 +620,12 @@ typedef struct EditorFuncs
 	void	(*editor_insert_text_block) (struct GeanyEditor *editor, const gchar *text,
 			 gint insert_pos, gint cursor_index, gint newline_indent_size,
 			 gboolean replace_newlines);
+
+	gint	(*editor_get_eol_char_mode) (struct GeanyEditor *editor);
+	gboolean (*editor_goto_pos) (struct GeanyEditor *editor, gint pos, gboolean mark);
+
+	const gchar* (*editor_find_snippet) (struct GeanyEditor *editor, const gchar *snippet_name);
+	void	(*editor_insert_snippet) (struct GeanyEditor *editor, gint pos, const gchar *snippet);
 }
 EditorFuncs;
 
