@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: printing.c 4518 2010-01-01 22:55:18Z eht16 $
+ * $Id: printing.c 5079 2010-07-01 16:22:45Z ntrel $
  */
 
 
@@ -491,6 +491,9 @@ static void begin_print(GtkPrintOperation *operation, GtkPrintContext *context,
 			dinfo->styles[i][BACK] = ROTATE_RGB(scintilla_send_message(
 				dinfo->doc->editor->sci, SCI_STYLEGETBACK, i, 0));
 		}
+		/* use white background color unless foreground is white to save ink */
+		if (dinfo->styles[i][FORE] != 0xffffff)
+			dinfo->styles[i][BACK] = 0xffffff;
 		dinfo->styles[i][BOLD] =
 			scintilla_send_message(dinfo->doc->editor->sci, SCI_STYLEGETBOLD, i, 0);
 		dinfo->styles[i][ITALIC] =
